@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -108,19 +103,16 @@ void ColorTip::paintEvent(QPaintEvent *event)
 {
     QTipLabel::paintEvent(event);
 
-    QPen pen;
-    pen.setWidth(1);
-    if (m_color.value() > 100)
-        pen.setColor(m_color.darker());
-    else
-        pen.setColor(m_color.lighter());
-
     QPainter painter(this);
-    painter.setPen(pen);
     painter.setBrush(m_color);
-    QRect r(0, 0, rect().width() - 1, rect().height() - 1);
-    painter.drawTiledPixmap(r, m_tilePixmap);
-    painter.drawRect(r);
+    painter.drawTiledPixmap(rect(), m_tilePixmap);
+
+    QPen pen;
+    pen.setColor(m_color.value() > 100 ? m_color.darker() : m_color.lighter());
+    pen.setJoinStyle(Qt::MiterJoin);
+    const QRectF borderRect = QRectF(rect()).adjusted(0.5, 0.5, -0.5, -0.5);
+    painter.setPen(pen);
+    painter.drawRect(borderRect);
 }
 
 TextTip::TextTip(QWidget *parent) : QTipLabel(parent)
@@ -135,9 +127,21 @@ TextTip::TextTip(QWidget *parent) : QTipLabel(parent)
     setWindowOpacity(style()->styleHint(QStyle::SH_ToolTipLabel_Opacity, 0, this) / 255.0);
 }
 
+static bool likelyContainsLink(const QString &s)
+{
+    return s.contains(QLatin1String("href"), Qt::CaseInsensitive);
+}
+
 void TextTip::setContent(const QVariant &content)
 {
     m_text = content.toString();
+    bool containsLink = likelyContainsLink(m_text);
+    setOpenExternalLinks(containsLink);
+}
+
+bool TextTip::isInteractive() const
+{
+    return likelyContainsLink(m_text);
 }
 
 void TextTip::configure(const QPoint &pos, QWidget *w)
@@ -162,11 +166,7 @@ void TextTip::configure(const QPoint &pos, QWidget *w)
     const int maxDesiredWidth = int(screenWidth * .5);
     if (tipWidth > maxDesiredWidth) {
         setWordWrap(true);
-        tipWidth = sizeHint().width();
-        // If the width is still too large (maybe due to some extremely long word which prevents
-        // wrapping), the tip is truncated according to the screen.
-        if (tipWidth > screenWidth)
-            tipWidth = screenWidth - 10;
+        tipWidth = maxDesiredWidth;
     }
 
     resize(tipWidth, heightForWidth(tipWidth) + extraHeight);
