@@ -24,11 +24,13 @@
 ****************************************************************************/
 
 #include "autotestconstants.h"
-#include "autotest_utils.h"
+#include "autotestplugin.h"
 #include "testcodeparser.h"
 #include "testframeworkmanager.h"
+#include "testsettings.h"
 
 #include <coreplugin/editormanager/editormanager.h>
+#include <coreplugin/messagemanager.h>
 #include <coreplugin/progressmanager/futureprogress.h>
 #include <coreplugin/progressmanager/progressmanager.h>
 #include <cpptools/cpptoolsconstants.h>
@@ -38,6 +40,7 @@
 #include <qmljstools/qmljsmodelmanager.h>
 
 #include <utils/algorithm.h>
+#include <utils/mapreduce.h>
 #include <utils/qtcassert.h>
 #include <utils/runextensions.h>
 
@@ -45,22 +48,18 @@
 #include <QFuture>
 #include <QFutureInterface>
 #include <QLoggingCategory>
-#include <QTimer>
 
 static Q_LOGGING_CATEGORY(LOG, "qtc.autotest.testcodeparser")
 
 namespace Autotest {
 namespace Internal {
 
+using namespace ProjectExplorer;
+
 TestCodeParser::TestCodeParser(TestTreeModel *parent)
     : QObject(parent),
       m_model(parent),
-      m_codeModelParsing(false),
-      m_fullUpdatePostponed(false),
-      m_partialUpdatePostponed(false),
-      m_dirty(false),
-      m_singleShotScheduled(false),
-      m_parserState(Disabled)
+      m_threadPool(new QThreadPool(this))
 {
     // connect to ProgressManager to postpone test parsing when CppModelManager is parsing
     auto progressManager = qobject_cast<Core::ProgressManager *>(Core::ProgressManager::instance());
@@ -77,6 +76,9 @@ TestCodeParser::TestCodeParser(TestTreeModel *parent)
         emit testParseResultReady(m_futureWatcher.resultAt(index));
     });
     connect(this, &TestCodeParser::parsingFinished, this, &TestCodeParser::releaseParserInternals);
+    m_reparseTimer.setSingleShot(true);
+    connect(&m_reparseTimer, &QTimer::timeout, this, &TestCodeParser::parsePostponedFiles);
+    m_threadPool->setMaxThreadCount(std::max(QThread::idealThreadCount()/4, 1));
 }
 
 TestCodeParser::~TestCodeParser()
@@ -95,30 +97,27 @@ void TestCodeParser::setState(State state)
         return;
     }
 
-    if ((state == Disabled || state == Idle)
-            && (m_parserState == PartialParse || m_parserState == FullParse)) {
+    if ((state == Idle) && (m_parserState == PartialParse || m_parserState == FullParse)) {
         qCDebug(LOG) << "Not setting state, parse is running";
         return;
     }
     m_parserState = state;
 
-    if (m_parserState == Disabled) {
-        m_fullUpdatePostponed = m_partialUpdatePostponed = false;
-        m_postponedFiles.clear();
-    } else if (m_parserState == Idle && ProjectExplorer::SessionManager::startupProject()) {
+    if (m_parserState == Idle && SessionManager::startupProject()) {
         if (m_fullUpdatePostponed || m_dirty) {
             emitUpdateTestTree();
         } else if (m_partialUpdatePostponed) {
             m_partialUpdatePostponed = false;
             qCDebug(LOG) << "calling scanForTests with postponed files (setState)";
-            scanForTests(m_postponedFiles.toList());
+            if (!m_reparseTimer.isActive())
+                scanForTests(m_postponedFiles.toList());
         }
     }
 }
 
 void TestCodeParser::syncTestFrameworks(const QVector<Core::Id> &frameworkIds)
 {
-    if (m_parserState != Disabled && m_parserState != Idle) {
+    if (m_parserState != Idle) {
         // there's a running parse
         m_fullUpdatePostponed = m_partialUpdatePostponed = false;
         m_postponedFiles.clear();
@@ -127,94 +126,89 @@ void TestCodeParser::syncTestFrameworks(const QVector<Core::Id> &frameworkIds)
     m_testCodeParsers.clear();
     TestFrameworkManager *frameworkManager = TestFrameworkManager::instance();
     qCDebug(LOG) << "Setting" << frameworkIds << "as current parsers";
-    foreach (const Core::Id &id, frameworkIds) {
+    for (const Core::Id &id : frameworkIds) {
         ITestParser *testParser = frameworkManager->testParserForTestFramework(id);
         QTC_ASSERT(testParser, continue);
         m_testCodeParsers.append(testParser);
     }
-    if (m_parserState != Disabled)
-        updateTestTree();
+    updateTestTree();
 }
 
-void TestCodeParser::emitUpdateTestTree()
+void TestCodeParser::emitUpdateTestTree(ITestParser *parser)
 {
+    if (m_testCodeParsers.isEmpty())
+        return;
     if (m_singleShotScheduled) {
+        if (m_updateParser && parser != m_updateParser)
+            m_updateParser = nullptr;
         qCDebug(LOG) << "not scheduling another updateTestTree";
         return;
     }
 
     qCDebug(LOG) << "adding singleShot";
     m_singleShotScheduled = true;
-    QTimer::singleShot(1000, this, &TestCodeParser::updateTestTree);
+    m_updateParser = parser;
+    QTimer::singleShot(1000, this, [this](){ updateTestTree(m_updateParser); });
 }
 
-void TestCodeParser::updateTestTree()
+void TestCodeParser::updateTestTree(ITestParser *parser)
 {
     m_singleShotScheduled = false;
     if (m_codeModelParsing) {
         m_fullUpdatePostponed = true;
         m_partialUpdatePostponed = false;
         m_postponedFiles.clear();
+        if (!parser || parser != m_updateParser)
+            m_updateParser = nullptr;
         return;
     }
 
-    if (!ProjectExplorer::SessionManager::startupProject())
+    if (!SessionManager::startupProject())
         return;
 
     m_fullUpdatePostponed = false;
-
-    emit aboutToPerformFullParse();
     qCDebug(LOG) << "calling scanForTests (updateTestTree)";
-    scanForTests();
+    scanForTests(QStringList(), parser);
+}
+
+static QStringList filterFiles(const QString &projectDir, const QStringList &files)
+{
+    const QSharedPointer<TestSettings> &settings = AutotestPlugin::instance()->settings();
+    const QSet<QString> &filters = settings->whiteListFilters.toSet(); // avoid duplicates
+    if (!settings->filterScan || filters.isEmpty())
+        return files;
+    QStringList finalResult;
+    for (const QString &file : files) {
+        // apply filter only below project directory if file is part of a project
+        const QString &fileToProcess = file.startsWith(projectDir)
+                ? file.mid(projectDir.size())
+                : file;
+        for (const QString &filter : filters) {
+            if (fileToProcess.contains(filter)) {
+                finalResult.push_back(file);
+                break;
+            }
+        }
+    }
+    return finalResult;
 }
 
 // used internally to indicate a parse that failed due to having triggered a parse for a file that
 // is not (yet) part of the CppModelManager's snapshot
 static bool parsingHasFailed;
 
-static bool checkDocumentForTestCode(QFutureInterface<TestParseResultPtr> &futureInterface,
-                                     const QString &fileName,
-                                     const QVector<ITestParser *> &parsers)
-{
-    foreach (ITestParser *currentParser, parsers) {
-        if (futureInterface.isCanceled())
-            return false;
-        if (currentParser->processDocument(futureInterface, fileName))
-            return true;
-    }
-    return false;
-}
-
-static void performParse(QFutureInterface<TestParseResultPtr> &futureInterface,
-                         const QStringList &list, const QVector<ITestParser *> &parsers)
-{
-    int progressValue = 0;
-    futureInterface.setProgressRange(0, list.size());
-    futureInterface.setProgressValue(progressValue);
-
-    foreach (const QString &file, list) {
-        if (futureInterface.isCanceled())
-            return;
-        futureInterface.setProgressValue(++progressValue);
-        if (!checkDocumentForTestCode(futureInterface, file, parsers)) {
-            parsingHasFailed |= !CppTools::CppModelManager::instance()->snapshot().contains(file)
-                && (CppTools::ProjectFile::classify(file) != CppTools::ProjectFile::Unclassified);
-        }
-    }
-    futureInterface.setProgressValue(list.size());
-}
-
 /****** threaded parsing stuff *******/
 
-void TestCodeParser::onDocumentUpdated(const QString &fileName)
+void TestCodeParser::onDocumentUpdated(const QString &fileName, bool isQmlFile)
 {
     if (m_codeModelParsing || m_fullUpdatePostponed)
         return;
 
-    ProjectExplorer::Project *project = ProjectExplorer::SessionManager::startupProject();
+    Project *project = SessionManager::startupProject();
     if (!project)
         return;
-    if (!project->files(ProjectExplorer::Project::SourceFiles).contains(fileName))
+    // Quick tests: qml files aren't necessarily listed inside project files
+    if (!isQmlFile && !SessionManager::projectContainsFile(project, Utils::FileName::fromString(fileName)))
         return;
 
     scanForTests(QStringList(fileName));
@@ -229,24 +223,25 @@ void TestCodeParser::onQmlDocumentUpdated(const QmlJS::Document::Ptr &document)
 {
     const QString fileName = document->fileName();
     if (!fileName.endsWith(".qbs"))
-        onDocumentUpdated(fileName);
+        onDocumentUpdated(fileName, true);
 }
 
-void TestCodeParser::onStartupProjectChanged(ProjectExplorer::Project *project)
+void TestCodeParser::onStartupProjectChanged(Project *project)
 {
     if (m_parserState == FullParse || m_parserState == PartialParse) {
         qCDebug(LOG) << "Canceling scanForTest (startup project changed)";
         Core::ProgressManager::instance()->cancelTasks(Constants::TASK_PARSE);
     }
+    emit aboutToPerformFullParse();
     if (project)
         emitUpdateTestTree();
 }
 
-void TestCodeParser::onProjectPartsUpdated(ProjectExplorer::Project *project)
+void TestCodeParser::onProjectPartsUpdated(Project *project)
 {
-    if (project != ProjectExplorer::SessionManager::startupProject())
+    if (project != SessionManager::startupProject())
         return;
-    if (m_codeModelParsing || m_parserState == Disabled)
+    if (m_codeModelParsing)
         m_fullUpdatePostponed = true;
     else
         emitUpdateTestTree();
@@ -267,6 +262,30 @@ bool TestCodeParser::postponed(const QStringList &fileList)
 {
     switch (m_parserState) {
     case Idle:
+        if (fileList.size() == 1) {
+            if (m_reparseTimerTimedOut)
+                return false;
+            switch (m_postponedFiles.size()) {
+            case 0:
+                m_postponedFiles.insert(fileList.first());
+                m_reparseTimer.setInterval(1000);
+                m_reparseTimer.start();
+                return true;
+            case 1:
+                if (m_postponedFiles.contains(fileList.first())) {
+                    m_reparseTimer.start();
+                    return true;
+                }
+                // intentional fall-through
+            default:
+                m_postponedFiles.insert(fileList.first());
+                m_reparseTimer.stop();
+                m_reparseTimer.setInterval(0);
+                m_reparseTimerTimedOut = false;
+                m_reparseTimer.start();
+                return true;
+            }
+        }
         return false;
     case PartialParse:
     case FullParse:
@@ -282,42 +301,49 @@ bool TestCodeParser::postponed(const QStringList &fileList)
             if (m_fullUpdatePostponed)
                 return true;
             // partial parse triggered, postpone or add current files to already postponed partial
-            foreach (const QString &file, fileList)
+            for (const QString &file : fileList)
                 m_postponedFiles.insert(file);
             m_partialUpdatePostponed = true;
         }
         return true;
-    case Disabled:
     case Shutdown:
         break;
     }
     QTC_ASSERT(false, return false); // should not happen at all
 }
 
-void TestCodeParser::scanForTests(const QStringList &fileList)
+static void parseFileForTests(const QVector<ITestParser *> &parsers,
+                              QFutureInterface<TestParseResultPtr> &futureInterface,
+                              const QString &fileName)
 {
-    if (m_parserState == Disabled || m_parserState == Shutdown) {
-        m_dirty = true;
-        if (fileList.isEmpty()) {
-            m_fullUpdatePostponed = true;
-            m_partialUpdatePostponed = false;
-            m_postponedFiles.clear();
-        } else if (!m_fullUpdatePostponed) {
-            m_partialUpdatePostponed = true;
-            foreach (const QString &file, fileList)
-                m_postponedFiles.insert(file);
-        }
-        return;
+    for (ITestParser *parser : parsers) {
+        if (futureInterface.isCanceled())
+            return;
+        if (parser->processDocument(futureInterface, fileName))
+            break;
     }
+}
+
+void TestCodeParser::scanForTests(const QStringList &fileList, ITestParser *parser)
+{
+    if (m_parserState == Shutdown || m_testCodeParsers.isEmpty())
+        return;
+    if (parser && !m_testCodeParsers.contains(parser))
+        return;
 
     if (postponed(fileList))
         return;
 
+    m_reparseTimer.stop();
+    m_reparseTimerTimedOut = false;
     m_postponedFiles.clear();
     bool isFullParse = fileList.isEmpty();
+    Project *project = SessionManager::startupProject();
+    if (!project)
+        return;
     QStringList list;
     if (isFullParse) {
-        list = ProjectExplorer::SessionManager::startupProject()->files(ProjectExplorer::Project::SourceFiles);
+        list = project->files(Project::SourceFiles);
         if (list.isEmpty()) {
             // at least project file should be there, but might happen if parsing current project
             // takes too long, especially when opening sessions holding multiple projects
@@ -338,18 +364,50 @@ void TestCodeParser::scanForTests(const QStringList &fileList)
     if (isFullParse) {
         // remove qml files as they will be found automatically by the referencing cpp file
         list = Utils::filtered(list, [] (const QString &fn) {
-            return !fn.endsWith(QLatin1String(".qml"));
+            return !fn.endsWith(".qml");
         });
-        m_model->markAllForRemoval();
+        if (parser)
+            TestFrameworkManager::instance()->rootNodeForTestFramework(parser->id())->markForRemovalRecursively(true);
+        else
+            m_model->markAllForRemoval();
+    } else if (parser) {
+        TestTreeItem *root = TestFrameworkManager::instance()->rootNodeForTestFramework(parser->id());
+        for (const QString &filePath : list)
+            root->markForRemovalRecursively(filePath);
     } else {
-        foreach (const QString &filePath, list)
+        for (const QString &filePath : list)
             m_model->markForRemoval(filePath);
     }
 
-    foreach (ITestParser *parser, m_testCodeParsers)
+    list = filterFiles(project->projectDirectory().toString(), list);
+    if (list.isEmpty()) {
+        if (isFullParse) {
+            Core::MessageManager::instance()->write(
+                        tr("AutoTest Plugin WARNING: No files left after filtering test scan "
+                           "folders. Check test filter settings."),
+                        Core::MessageManager::Flash);
+        }
+        onFinished();
+        return;
+    }
+
+    // use only a single parser or all current active?
+    QVector<ITestParser *> codeParsers;
+    if (parser)
+        codeParsers.append(parser);
+    else
+        codeParsers.append(m_testCodeParsers);
+    qCDebug(LOG) << QDateTime::currentDateTime().toString("hh:mm:ss.zzz") << "StartParsing";
+    for (ITestParser *parser : codeParsers)
         parser->init(list);
 
-    QFuture<TestParseResultPtr> future = Utils::runAsync(&performParse, list, m_testCodeParsers);
+    QFuture<TestParseResultPtr> future = Utils::map(list,
+        [this, codeParsers](QFutureInterface<TestParseResultPtr> &fi, const QString &file) {
+            parseFileForTests(codeParsers, fi, file);
+        },
+        Utils::MapReduceOption::Unordered,
+        m_threadPool,
+        QThread::LowestPriority);
     m_futureWatcher.setFuture(future);
     if (list.size() > 5) {
         Core::ProgressManager::addTask(future, tr("Scanning for Tests"),
@@ -365,6 +423,7 @@ void TestCodeParser::onTaskStarted(Core::Id type)
             m_fullUpdatePostponed = m_parserState == FullParse;
             m_partialUpdatePostponed = !m_fullUpdatePostponed;
             qCDebug(LOG) << "Canceling scan for test (CppModelParsing started)";
+            parsingHasFailed = true;
             Core::ProgressManager::instance()->cancelTasks(Constants::TASK_PARSE);
         }
     }
@@ -372,6 +431,10 @@ void TestCodeParser::onTaskStarted(Core::Id type)
 
 void TestCodeParser::onAllTasksFinished(Core::Id type)
 {
+    // if we cancel parsing ensure that progress animation is canceled as well
+    if (type == Constants::TASK_PARSE && parsingHasFailed)
+        emit parsingFailed();
+
     // only CPP parsing is relevant as we trigger Qml parsing internally anyway
     if (type != CppTools::Constants::TASK_INDEX)
         return;
@@ -383,11 +446,14 @@ void TestCodeParser::onAllTasksFinished(Core::Id type)
 
 void TestCodeParser::onFinished()
 {
+    if (m_futureWatcher.isCanceled())
+        parsingHasFailed = true;
     switch (m_parserState) {
     case PartialParse:
         qCDebug(LOG) << "setting state to Idle (onFinished, PartialParse)";
         m_parserState = Idle;
         onPartialParsingFinished();
+        qCDebug(LOG) << QDateTime::currentDateTime().toString("hh:mm:ss.zzz") << "PartParsingFin";
         break;
     case FullParse:
         qCDebug(LOG) << "setting state to Idle (onFinished, FullParse)";
@@ -398,14 +464,12 @@ void TestCodeParser::onFinished()
         } else {
             qCDebug(LOG) << "emitting parsingFinished"
                          << "(onFinished, FullParse, nothing postponed, parsing succeeded)";
+            m_updateParser = nullptr;
             emit parsingFinished();
+            qCDebug(LOG) << QDateTime::currentDateTime().toString("hh:mm:ss.zzz") << "ParsingFin";
         }
         m_dirty = false;
         break;
-    case Disabled: // can happen if all Test related widgets become hidden while parsing
-        qCDebug(LOG) << "emitting parsingFinished (onFinished, Disabled)";
-        emit parsingFinished();
-        break;
     case Shutdown:
         qCDebug(LOG) << "Shutdown complete - not emitting parsingFinished (onFinished)";
         break;
@@ -423,19 +487,23 @@ void TestCodeParser::onPartialParsingFinished()
     if (m_fullUpdatePostponed) {
         m_fullUpdatePostponed = false;
         qCDebug(LOG) << "calling updateTestTree (onPartialParsingFinished)";
-        updateTestTree();
+        updateTestTree(m_updateParser);
     } else if (m_partialUpdatePostponed) {
         m_partialUpdatePostponed = false;
         qCDebug(LOG) << "calling scanForTests with postponed files (onPartialParsingFinished)";
-        scanForTests(m_postponedFiles.toList());
+        if (!m_reparseTimer.isActive())
+            scanForTests(m_postponedFiles.toList());
     } else {
         m_dirty |= m_codeModelParsing;
         if (m_dirty) {
             emit parsingFailed();
+            qCDebug(LOG) << QDateTime::currentDateTime().toString("hh:mm:ss.zzz") << "ParsingFail";
         } else if (!m_singleShotScheduled) {
             qCDebug(LOG) << "emitting parsingFinished"
                          << "(onPartialParsingFinished, nothing postponed, not dirty)";
+            m_updateParser = nullptr;
             emit parsingFinished();
+            qCDebug(LOG) << QDateTime::currentDateTime().toString("hh:mm:ss.zzz") << "ParsingFin";
         } else {
             qCDebug(LOG) << "not emitting parsingFinished"
                          << "(on PartialParsingFinished, singleshot scheduled)";
@@ -443,6 +511,12 @@ void TestCodeParser::onPartialParsingFinished()
     }
 }
 
+void TestCodeParser::parsePostponedFiles()
+{
+    m_reparseTimerTimedOut = true;
+    scanForTests(m_postponedFiles.toList());
+}
+
 void TestCodeParser::releaseParserInternals()
 {
     for (ITestParser *parser : m_testCodeParsers)
