@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,27 +9,22 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "outputpanemanager.h"
 #include "outputpane.h"
-#include "coreconstants.h"
 #include "findplaceholder.h"
 
 #include "icore.h"
@@ -38,14 +33,20 @@
 
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/actioncontainer.h>
+#include <coreplugin/actionmanager/command.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/ieditor.h>
 
 #include <extensionsystem/pluginmanager.h>
 
+#include <utils/algorithm.h>
 #include <utils/hostosinfo.h>
 #include <utils/styledbar.h>
+#include <utils/stylehelper.h>
+#include <utils/proxyaction.h>
 #include <utils/qtcassert.h>
+#include <utils/theme/theme.h>
+#include <utils/utilsicons.h>
 
 #include <QDebug>
 
@@ -62,12 +63,20 @@
 #include <QToolButton>
 #include <QTimeLine>
 
+using namespace Utils;
+
 namespace Core {
 namespace Internal {
 
 static char outputPaneSettingsKeyC[] = "OutputPaneVisibility";
 static char outputPaneIdKeyC[] = "id";
 static char outputPaneVisibleKeyC[] = "visible";
+static const int buttonBorderWidth = 3;
+
+static int numberAreaWidth()
+{
+    return creatorTheme()->flag(Theme::FlatToolBars) ? 15 : 19;
+}
 
 ////
 // OutputPaneManager
@@ -75,11 +84,6 @@ static char outputPaneVisibleKeyC[] = "visible";
 
 static OutputPaneManager *m_instance = 0;
 
-bool comparePanes(IOutputPane *p1, IOutputPane *p2)
-{
-    return p1->priorityInStatusBar() > p2->priorityInStatusBar();
-}
-
 void OutputPaneManager::create()
 {
    m_instance = new OutputPaneManager;
@@ -104,8 +108,17 @@ void OutputPaneManager::updateStatusButtons(bool visible)
     QTC_ASSERT(m_panes.size() == m_buttons.size(), return);
     m_buttons.at(idx)->setChecked(visible);
     m_panes.at(idx)->visibilityChanged(visible);
-    OutputPanePlaceHolder *ph = OutputPanePlaceHolder::getCurrent();
-    m_minMaxAction->setVisible(ph && ph->canMaximizeOrMinimize());
+}
+
+void OutputPaneManager::updateMaximizeButton(bool maximized)
+{
+    if (maximized) {
+        m_instance->m_minMaxAction->setIcon(m_instance->m_minimizeIcon);
+        m_instance->m_minMaxAction->setText(tr("Minimize Output Pane"));
+    } else {
+        m_instance->m_minMaxAction->setIcon(m_instance->m_maximizeIcon);
+        m_instance->m_minMaxAction->setText(tr("Maximize Output Pane"));
+    }
 }
 
 OutputPaneManager::OutputPaneManager(QWidget *parent) :
@@ -119,48 +132,47 @@ OutputPaneManager::OutputPaneManager(QWidget *parent) :
     m_prevAction(0),
     m_outputWidgetPane(new QStackedWidget),
     m_opToolBarWidgets(new QStackedWidget),
-    m_minimizeIcon(QLatin1String(":/core/images/arrowdown.png")),
-    m_maximizeIcon(QLatin1String(":/core/images/arrowup.png")),
-    m_maximised(false),
-    m_outputPaneHeight(0)
+    m_minimizeIcon(Utils::Icons::ARROW_DOWN.icon()),
+    m_maximizeIcon(Utils::Icons::ARROW_UP.icon()),
+    m_outputPaneHeightSetting(0)
 {
     setWindowTitle(tr("Output"));
 
     m_titleLabel->setContentsMargins(5, 0, 5, 0);
 
     m_clearAction = new QAction(this);
-    m_clearAction->setIcon(QIcon(QLatin1String(Constants::ICON_CLEAN_PANE)));
+    m_clearAction->setIcon(Utils::Icons::CLEAN.icon());
     m_clearAction->setText(tr("Clear"));
-    connect(m_clearAction, SIGNAL(triggered()), this, SLOT(clearPage()));
+    connect(m_clearAction, &QAction::triggered, this, &OutputPaneManager::clearPage);
 
     m_nextAction = new QAction(this);
-    m_nextAction->setIcon(QIcon(QLatin1String(Constants::ICON_NEXT)));
+    m_nextAction->setIcon(Utils::Icons::NEXT.icon());
     m_nextAction->setText(tr("Next Item"));
-    connect(m_nextAction, SIGNAL(triggered()), this, SLOT(slotNext()));
+    connect(m_nextAction, &QAction::triggered, this, &OutputPaneManager::slotNext);
 
     m_prevAction = new QAction(this);
-    m_prevAction->setIcon(QIcon(QLatin1String(Constants::ICON_PREV)));
+    m_prevAction->setIcon(Utils::Icons::PREV.icon());
     m_prevAction->setText(tr("Previous Item"));
-    connect(m_prevAction, SIGNAL(triggered()), this, SLOT(slotPrev()));
+    connect(m_prevAction, &QAction::triggered, this, &OutputPaneManager::slotPrev);
 
     m_minMaxAction = new QAction(this);
     m_minMaxAction->setIcon(m_maximizeIcon);
     m_minMaxAction->setText(tr("Maximize Output Pane"));
 
-    m_closeButton->setIcon(QIcon(QLatin1String(Constants::ICON_CLOSE_DOCUMENT)));
-    connect(m_closeButton, SIGNAL(clicked()), this, SLOT(slotHide()));
+    m_closeButton->setIcon(Icons::CLOSE_SPLIT_BOTTOM.icon());
+    connect(m_closeButton, &QAbstractButton::clicked, this, &OutputPaneManager::slotHide);
 
-    connect(ICore::instance(), SIGNAL(saveSettingsRequested()), this, SLOT(saveSettings()));
+    connect(ICore::instance(), &ICore::saveSettingsRequested, this, &OutputPaneManager::saveSettings);
 
     QVBoxLayout *mainlayout = new QVBoxLayout;
     mainlayout->setSpacing(0);
     mainlayout->setMargin(0);
-    m_toolBar = new Utils::StyledBar;
+    m_toolBar = new StyledBar;
     QHBoxLayout *toolLayout = new QHBoxLayout(m_toolBar);
     toolLayout->setMargin(0);
     toolLayout->setSpacing(0);
     toolLayout->addWidget(m_titleLabel);
-    toolLayout->addWidget(new Utils::StyledSeparator);
+    toolLayout->addWidget(new StyledSeparator);
     m_clearButton = new QToolButton;
     toolLayout->addWidget(m_clearButton);
     m_prevToolButton = new QToolButton;
@@ -172,13 +184,14 @@ OutputPaneManager::OutputPaneManager(QWidget *parent) :
     toolLayout->addWidget(m_closeButton);
     mainlayout->addWidget(m_toolBar);
     mainlayout->addWidget(m_outputWidgetPane, 10);
-    mainlayout->addWidget(new Core::FindToolBarPlaceHolder(this));
+    mainlayout->addWidget(new FindToolBarPlaceHolder(this));
     setLayout(mainlayout);
 
     m_buttonsWidget = new QWidget;
     m_buttonsWidget->setLayout(new QHBoxLayout);
     m_buttonsWidget->layout()->setContentsMargins(5,0,0,0);
-    m_buttonsWidget->layout()->setSpacing(4);
+    m_buttonsWidget->layout()->setSpacing(
+            creatorTheme()->flag(Theme::FlatToolBars) ? 9 : 4);
 }
 
 OutputPaneManager::~OutputPaneManager()
@@ -190,17 +203,19 @@ QWidget *OutputPaneManager::buttonsWidget()
     return m_buttonsWidget;
 }
 
-// Return shortcut as Ctrl+<number>
-static inline int paneShortCut(int number)
+// Return shortcut as Alt+<number> or Cmd+<number> if number is a non-zero digit
+static inline QKeySequence paneShortCut(int number)
 {
-    const int modifier = Utils::HostOsInfo::isMacHost() ? Qt::CTRL : Qt::ALT;
-    return modifier | (Qt::Key_0 + number);
+    if (number < 1 || number > 9)
+        return QKeySequence();
+
+    const int modifier = HostOsInfo::isMacHost() ? Qt::CTRL : Qt::ALT;
+    return QKeySequence(modifier | (Qt::Key_0 + number));
 }
 
 void OutputPaneManager::init()
 {
     ActionContainer *mwindow = ActionManager::actionContainer(Constants::M_WINDOW);
-    const Context globalContext(Constants::C_GLOBAL);
 
     // Window->Output Panes
     ActionContainer *mpanes = ActionManager::createMenu(Constants::M_WINDOW_PANES);
@@ -211,35 +226,40 @@ void OutputPaneManager::init()
 
     Command *cmd;
 
-    cmd = ActionManager::registerAction(m_clearAction, "Coreplugin.OutputPane.clear", globalContext);
+    cmd = ActionManager::registerAction(m_clearAction, "Coreplugin.OutputPane.clear");
     m_clearButton->setDefaultAction(cmd->action());
+    m_clearButton->setIcon(Utils::Icons::CLEAN_TOOLBAR.icon());
     mpanes->addAction(cmd, "Coreplugin.OutputPane.ActionsGroup");
 
-    cmd = ActionManager::registerAction(m_prevAction, "Coreplugin.OutputPane.previtem", globalContext);
+    cmd = ActionManager::registerAction(m_prevAction, "Coreplugin.OutputPane.previtem");
     cmd->setDefaultKeySequence(QKeySequence(tr("Shift+F6")));
-    m_prevToolButton->setDefaultAction(cmd->action());
+    m_prevToolButton->setDefaultAction(
+                ProxyAction::proxyActionWithIcon(cmd->action(), Utils::Icons::PREV_TOOLBAR.icon()));
     mpanes->addAction(cmd, "Coreplugin.OutputPane.ActionsGroup");
 
-    cmd = ActionManager::registerAction(m_nextAction, "Coreplugin.OutputPane.nextitem", globalContext);
-    m_nextToolButton->setDefaultAction(cmd->action());
+    cmd = ActionManager::registerAction(m_nextAction, "Coreplugin.OutputPane.nextitem");
+    m_nextToolButton->setDefaultAction(
+                ProxyAction::proxyActionWithIcon(cmd->action(), Utils::Icons::NEXT_TOOLBAR.icon()));
     cmd->setDefaultKeySequence(QKeySequence(tr("F6")));
     mpanes->addAction(cmd, "Coreplugin.OutputPane.ActionsGroup");
 
-    cmd = ActionManager::registerAction(m_minMaxAction, "Coreplugin.OutputPane.minmax", globalContext);
-    cmd->setDefaultKeySequence(QKeySequence(UseMacShortcuts ? tr("Ctrl+9") : tr("Alt+9")));
+    cmd = ActionManager::registerAction(m_minMaxAction, "Coreplugin.OutputPane.minmax");
+    cmd->setDefaultKeySequence(QKeySequence(UseMacShortcuts ? tr("Ctrl+Shift+9") : tr("Alt+Shift+9")));
     cmd->setAttribute(Command::CA_UpdateText);
     cmd->setAttribute(Command::CA_UpdateIcon);
     mpanes->addAction(cmd, "Coreplugin.OutputPane.ActionsGroup");
-    connect(m_minMaxAction, SIGNAL(triggered()), this, SLOT(slotMinMax()));
+    connect(m_minMaxAction, &QAction::triggered, this, &OutputPaneManager::toggleMaximized);
     m_minMaxButton->setDefaultAction(cmd->action());
 
-    mpanes->addSeparator(globalContext, "Coreplugin.OutputPane.ActionsGroup");
+    mpanes->addSeparator("Coreplugin.OutputPane.ActionsGroup");
 
     QFontMetrics titleFm = m_titleLabel->fontMetrics();
     int minTitleWidth = 0;
 
     m_panes = ExtensionSystem::PluginManager::getObjects<IOutputPane>();
-    qSort(m_panes.begin(), m_panes.end(), &comparePanes);
+    Utils::sort(m_panes, [](IOutputPane *p1, IOutputPane *p2) {
+        return p1->priorityInStatusBar() > p2->priorityInStatusBar();
+    });
     const int n = m_panes.size();
 
     int shortcutNumber = 1;
@@ -249,12 +269,15 @@ void OutputPaneManager::init()
         const int idx = m_outputWidgetPane->addWidget(outPane->outputWidget(this));
         QTC_CHECK(idx == i);
 
-        connect(outPane, SIGNAL(showPage(int)), this, SLOT(showPage(int)));
-        connect(outPane, SIGNAL(hidePage()), this, SLOT(slotHide()));
-        connect(outPane, SIGNAL(togglePage(int)), this, SLOT(togglePage(int)));
-        connect(outPane, SIGNAL(navigateStateUpdate()), this, SLOT(updateNavigateState()));
-        connect(outPane, SIGNAL(flashButton()), this, SLOT(flashButton()));
-        connect(outPane, SIGNAL(setBadgeNumber(int)), this, SLOT(setBadgeNumber(int)));
+        connect(outPane, &IOutputPane::showPage, this, [this, outPane](int flags) {
+            showPage(findIndexForPage(outPane), flags);
+        });
+        connect(outPane, &IOutputPane::hidePage, this, &OutputPaneManager::slotHide);
+        connect(outPane, &IOutputPane::togglePage, this, &OutputPaneManager::togglePage);
+        connect(outPane, &IOutputPane::navigateStateUpdate,
+                this, &OutputPaneManager::updateNavigateState);
+        connect(outPane, &IOutputPane::flashButton, this, &OutputPaneManager::flashButton);
+        connect(outPane, &IOutputPane::setBadgeNumber, this, &OutputPaneManager::setBadgeNumber);
 
         QWidget *toolButtonsContainer = new QWidget(m_opToolBarWidgets);
         QHBoxLayout *toolButtonsLayout = new QHBoxLayout;
@@ -273,31 +296,33 @@ void OutputPaneManager::init()
         suffix.remove(QLatin1Char(' '));
         const Id id = baseId.withSuffix(suffix);
         QAction *action = new QAction(outPane->displayName(), this);
-        Command *cmd = ActionManager::registerAction(action, id, globalContext);
+        Command *cmd = ActionManager::registerAction(action, id);
 
         mpanes->addAction(cmd, "Coreplugin.OutputPane.PanesGroup");
         m_actions.append(action);
         m_ids.append(id);
 
-        cmd->setDefaultKeySequence(QKeySequence(paneShortCut(shortcutNumber)));
+        cmd->setDefaultKeySequence(paneShortCut(shortcutNumber));
         OutputPaneToggleButton *button = new OutputPaneToggleButton(shortcutNumber, outPane->displayName(),
                                                                     cmd->action());
         ++shortcutNumber;
         m_buttonsWidget->layout()->addWidget(button);
         m_buttons.append(button);
-        connect(button, SIGNAL(clicked()), this, SLOT(buttonTriggered()));
+        connect(button, &QAbstractButton::clicked, this, [this, button]() {
+            buttonTriggered(m_buttons.indexOf(button));
+         });
 
         bool visible = outPane->priorityInStatusBar() != -1;
         button->setVisible(visible);
         m_buttonVisibility.insert(id, visible);
 
-        connect(action, SIGNAL(triggered()), this, SLOT(shortcutTriggered()));
+        connect(action, &QAction::triggered, this, &OutputPaneManager::shortcutTriggered);
     }
 
     m_titleLabel->setMinimumWidth(minTitleWidth + m_titleLabel->contentsMargins().left()
                                   + m_titleLabel->contentsMargins().right());
     m_buttonsWidget->layout()->addWidget(m_manageButton);
-    connect(m_manageButton, SIGNAL(clicked()), this, SLOT(popupMenu()));
+    connect(m_manageButton, &QAbstractButton::clicked, this, &OutputPaneManager::popupMenu);
 
     readSettings();
 }
@@ -308,7 +333,7 @@ void OutputPaneManager::shortcutTriggered()
     QTC_ASSERT(action, return);
     int idx = m_actions.indexOf(action);
     QTC_ASSERT(idx != -1, return);
-    Core::IOutputPane *outputPane = m_panes.at(idx);
+    IOutputPane *outputPane = m_panes.at(idx);
     // Now check the special case, the output window is already visible,
     // we are already on that page but the outputpane doesn't have focus
     // then just give it focus.
@@ -326,29 +351,24 @@ void OutputPaneManager::shortcutTriggered()
     }
 }
 
-bool OutputPaneManager::isMaximized()const
+int OutputPaneManager::outputPaneHeightSetting()
 {
-    return m_maximised;
+    return m_instance->m_outputPaneHeightSetting;
 }
 
-void OutputPaneManager::slotMinMax()
+void OutputPaneManager::setOutputPaneHeightSetting(int value)
+{
+    m_instance->m_outputPaneHeightSetting = value;
+}
+
+void OutputPaneManager::toggleMaximized()
 {
     OutputPanePlaceHolder *ph = OutputPanePlaceHolder::getCurrent();
     QTC_ASSERT(ph, return);
 
     if (!ph->isVisible()) // easier than disabling/enabling the action
         return;
-    m_maximised = !m_maximised;
-    ph->maximizeOrMinimize(m_maximised);
-    m_minMaxAction->setIcon(m_maximised ? m_minimizeIcon : m_maximizeIcon);
-    m_minMaxAction->setText(m_maximised ? tr("Minimize Output Pane")
-                                            : tr("Maximize Output Pane"));
-}
-
-void OutputPaneManager::buttonTriggered()
-{
-    OutputPaneToggleButton *button = qobject_cast<OutputPaneToggleButton *>(sender());
-    buttonTriggered(m_buttons.indexOf(button));
+    ph->setMaximized(!ph->isMaximized());
 }
 
 void OutputPaneManager::buttonTriggered(int idx)
@@ -378,7 +398,7 @@ void OutputPaneManager::readSettings()
             m_buttons.at(i)->setVisible(m_buttonVisibility.value(m_ids.at(i)));
     }
 
-    m_outputPaneHeight = settings->value(QLatin1String("OutputPanePlaceHolder/Height"), 0).toInt();
+    m_outputPaneHeightSetting = settings->value(QLatin1String("OutputPanePlaceHolder/Height"), 0).toInt();
 }
 
 void OutputPaneManager::slotNext()
@@ -408,7 +428,7 @@ void OutputPaneManager::slotHide()
         QTC_ASSERT(idx >= 0, return);
         m_buttons.at(idx)->setChecked(false);
         m_panes.value(idx)->visibilityChanged(false);
-        if (IEditor *editor = Core::EditorManager::currentEditor()) {
+        if (IEditor *editor = EditorManager::currentEditor()) {
             QWidget *w = editor->widget()->focusWidget();
             if (!w)
                 w = editor->widget();
@@ -456,13 +476,6 @@ void OutputPaneManager::setBadgeNumber(int number)
         m_buttons.value(idx)->setIconBadgeNumber(number);
 }
 
-// Slot connected to showPage signal of each page
-void OutputPaneManager::showPage(int flags)
-{
-    int idx = findIndexForPage(qobject_cast<IOutputPane*>(sender()));
-    showPage(idx, flags);
-}
-
 void OutputPaneManager::showPage(int idx, int flags)
 {
     QTC_ASSERT(idx >= 0, return);
@@ -489,12 +502,12 @@ void OutputPaneManager::showPage(int idx, int flags)
         ensurePageVisible(idx);
         IOutputPane *out = m_panes.at(idx);
         out->visibilityChanged(true);
-        if (flags & IOutputPane::WithFocus && out->canFocus()) {
-            out->setFocus();
+        if (flags & IOutputPane::WithFocus) {
+            if (out->canFocus())
+                out->setFocus();
             ICore::raiseWindow(m_outputWidgetPane);
         }
 
-        ph->setDefaultHeight(m_outputPaneHeight);
         if (flags & IOutputPane::EnsureSizeHint)
             ph->ensureSizeHintAsMinimum();
     }
@@ -515,13 +528,6 @@ void OutputPaneManager::focusInEvent(QFocusEvent *e)
         w->setFocus(e->reason());
 }
 
-void OutputPaneManager::resizeEvent(QResizeEvent *e)
-{
-    if (e->size().height() == 0)
-        return;
-    m_outputPaneHeight = e->size().height();
-}
-
 void OutputPaneManager::setCurrentIndex(int idx)
 {
     static int lastIndex = -1;
@@ -590,7 +596,11 @@ void OutputPaneManager::saveSettings() const
                            m_buttonVisibility.value(m_ids.at(i)));
     }
     settings->endArray();
-    settings->setValue(QLatin1String("OutputPanePlaceHolder/Height"), m_outputPaneHeight);
+    int heightSetting = m_outputPaneHeightSetting;
+    // update if possible
+    if (OutputPanePlaceHolder *curr = OutputPanePlaceHolder::getCurrent())
+        heightSetting = curr->nonMaximizedSize();
+    settings->setValue(QLatin1String("OutputPanePlaceHolder/Height"), heightSetting);
 }
 
 void OutputPaneManager::clearPage()
@@ -612,20 +622,6 @@ int OutputPaneManager::currentIndex() const
 //
 ///////////////////////////////////////////////////////////////////////
 
-static QString buttonStyleSheet()
-{
-    QString styleSheet = QLatin1String("QToolButton { border-image: url(:/core/images/panel_button.png) 2 2 2 19;"
-            " border-width: 2px 2px 2px 19px; padding-left: -17; padding-right: 4 } "
-            "QToolButton:checked { border-image: url(:/core/images/panel_button_checked.png) 2 2 2 19 } "
-            "QToolButton::menu-indicator { width:0; height:0 }");
-    if (!Utils::HostOsInfo::isMacHost()) { // Mac UIs usually don't hover
-        styleSheet += QLatin1String("QToolButton:checked:hover { border-image: url(:/core/images/panel_button_checked_hover.png) 2 2 2 19 } "
-                "QToolButton:pressed:hover { border-image: url(:/core/images/panel_button_pressed.png) 2 2 2 19 } "
-                "QToolButton:hover { border-image: url(:/core/images/panel_button_hover.png) 2 2 2 19 } ");
-    }
-    return styleSheet;
-}
-
 OutputPaneToggleButton::OutputPaneToggleButton(int number, const QString &text,
                                                QAction *action, QWidget *parent)
     : QToolButton(parent)
@@ -638,29 +634,21 @@ OutputPaneToggleButton::OutputPaneToggleButton(int number, const QString &text,
     setCheckable(true);
     QFont fnt = QApplication::font();
     setFont(fnt);
-    setStyleSheet(buttonStyleSheet());
     if (m_action)
-        connect(m_action, SIGNAL(changed()), this, SLOT(updateToolTip()));
+        connect(m_action, &QAction::changed, this, &OutputPaneToggleButton::updateToolTip);
 
     m_flashTimer->setDirection(QTimeLine::Forward);
     m_flashTimer->setCurveShape(QTimeLine::SineCurve);
     m_flashTimer->setFrameRange(0, 92);
-    connect(m_flashTimer, SIGNAL(valueChanged(qreal)), this, SLOT(update()));
-    connect(m_flashTimer, SIGNAL(finished()), this, SLOT(update()));
-
-    m_label = new QLabel(this);
-    fnt.setBold(true);
-    fnt.setPixelSize(11);
-    m_label->setFont(fnt);
-    m_label->setAlignment(Qt::AlignCenter);
-    m_label->setStyleSheet(QLatin1String("background-color: #818181; color: white; border-radius: 6; padding-left: 4; padding-right: 4;"));
-    m_label->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
-    m_label->hide();
+    auto updateSlot = static_cast<void (QWidget::*)()>(&QWidget::update);
+    connect(m_flashTimer, &QTimeLine::valueChanged, this, updateSlot);
+    connect(m_flashTimer, &QTimeLine::finished, this, updateSlot);
+    updateToolTip();
 }
 
 void OutputPaneToggleButton::updateToolTip()
 {
-    Q_ASSERT(m_action);
+    QTC_ASSERT(m_action, return);
     setToolTip(m_action->toolTip());
 }
 
@@ -670,46 +658,89 @@ QSize OutputPaneToggleButton::sizeHint() const
 
     QSize s = fontMetrics().size(Qt::TextSingleLine, m_text);
 
-    // Expand to account for border image set by stylesheet above
-    s.rwidth() += 19 + 5 + 2;
-    s.rheight() += 2 + 2;
+    // Expand to account for border image
+    s.rwidth() += numberAreaWidth() + 1 + buttonBorderWidth + buttonBorderWidth;
 
-    if (!m_label->text().isNull())
-        s.rwidth() += m_label->width();
+    if (!m_badgeNumberLabel.text().isNull())
+        s.rwidth() += m_badgeNumberLabel.sizeHint().width() + 1;
 
     return s.expandedTo(QApplication::globalStrut());
 }
 
-void OutputPaneToggleButton::resizeEvent(QResizeEvent *event)
+void OutputPaneToggleButton::paintEvent(QPaintEvent*)
 {
-    QToolButton::resizeEvent(event);
-    if (!m_label->text().isNull()) {
-        m_label->move(width() - m_label->width() - 3,  (height() - m_label->height() + 1) / 2);
-        m_label->show();
-    }
-}
-
-void OutputPaneToggleButton::paintEvent(QPaintEvent *event)
-{
-    // For drawing the style sheet stuff
-    QToolButton::paintEvent(event);
-
     const QFontMetrics fm = fontMetrics();
     const int baseLine = (height() - fm.height() + 1) / 2 + fm.ascent();
     const int numberWidth = fm.width(m_number);
 
     QPainter p(this);
-    if (m_flashTimer->state() == QTimeLine::Running) {
-        p.setPen(Qt::transparent);
-        p.fillRect(rect().adjusted(19, 1, -1, -1), QBrush(QColor(255,0,0, m_flashTimer->currentFrame())));
+
+    QStyleOption styleOption;
+    styleOption.initFrom(this);
+    const bool hovered = !HostOsInfo::isMacHost() && (styleOption.state & QStyle::State_MouseOver);
+
+    if (creatorTheme()->flag(Theme::FlatToolBars)) {
+        Theme::Color c = Theme::BackgroundColorDark;
+
+        if (hovered)
+            c = Theme::BackgroundColorHover;
+        else if (isDown() || isChecked())
+            c = Theme::BackgroundColorSelected;
+
+        if (c != Theme::BackgroundColorDark)
+            p.fillRect(rect(), creatorTheme()->color(c));
+    } else {
+        const QImage *image = 0;
+        if (isDown()) {
+            static const QImage pressed(
+                        StyleHelper::dpiSpecificImageFile(":/utils/images/panel_button_pressed.png"));
+            image = &pressed;
+        } else if (isChecked()) {
+            if (hovered) {
+                static const QImage checkedHover(
+                            StyleHelper::dpiSpecificImageFile(":/utils/images/panel_button_checked_hover.png"));
+                image = &checkedHover;
+            } else {
+                static const QImage checked(
+                            StyleHelper::dpiSpecificImageFile(":/utils/images/panel_button_checked.png"));
+                image = &checked;
+            }
+        } else {
+            if (hovered) {
+                static const QImage hover(
+                            StyleHelper::dpiSpecificImageFile(":/utils/images/panel_button_hover.png"));
+                image = &hover;
+            } else {
+                static const QImage button(
+                            StyleHelper::dpiSpecificImageFile(":/utils/images/panel_button.png"));
+                image = &button;
+            }
+        }
+        if (image)
+            StyleHelper::drawCornerImage(*image, &p, rect(), numberAreaWidth(), buttonBorderWidth, buttonBorderWidth, buttonBorderWidth);
     }
+
+    if (m_flashTimer->state() == QTimeLine::Running)
+    {
+        QColor c = creatorTheme()->color(Theme::OutputPaneButtonFlashColor);
+        c.setAlpha (m_flashTimer->currentFrame());
+        QRect r = creatorTheme()->flag(Theme::FlatToolBars)
+                  ? rect() : rect().adjusted(numberAreaWidth(), 1, -1, -1);
+        p.fillRect(r, c);
+    }
+
     p.setFont(font());
-    p.setPen(Qt::white);
-    p.drawText((20 - numberWidth) / 2, baseLine, m_number);
+    p.setPen(creatorTheme()->color(Theme::OutputPaneToggleButtonTextColorChecked));
+    p.drawText((numberAreaWidth() - numberWidth) / 2, baseLine, m_number);
     if (!isChecked())
-        p.setPen(Qt::black);
-    int leftPart = 22;
-    int labelWidth = m_label->isVisible() ? m_label->width() + 3 : 0;
+        p.setPen(creatorTheme()->color(Theme::OutputPaneToggleButtonTextColorUnchecked));
+    int leftPart = numberAreaWidth() + buttonBorderWidth;
+    int labelWidth = 0;
+    if (!m_badgeNumberLabel.text().isEmpty()) {
+        const QSize labelSize = m_badgeNumberLabel.sizeHint();
+        labelWidth = labelSize.width() + 3;
+        m_badgeNumberLabel.paint(&p, width() - labelWidth, (height() - labelSize.height()) / 2, isChecked());
+    }
     p.drawText(leftPart, baseLine, fm.elidedText(m_text, Qt::ElideRight, width() - leftPart - 1 - labelWidth));
 }
 
@@ -718,13 +749,6 @@ void OutputPaneToggleButton::checkStateSet()
     //Stop flashing when button is checked
     QToolButton::checkStateSet();
     m_flashTimer->stop();
-
-    if (isChecked())
-        m_label->setStyleSheet(QLatin1String("background-color: #e1e1e1; color: #606060; "
-                                             "border-radius: 6; padding-left: 4; padding-right: 4;"));
-    else
-        m_label->setStyleSheet(QLatin1String("background-color: #818181; color: white; border-radius: 6; "
-                                             "padding-left: 4; padding-right: 4;"));
 }
 
 void OutputPaneToggleButton::flash(int count)
@@ -741,21 +765,8 @@ void OutputPaneToggleButton::flash(int count)
 
 void OutputPaneToggleButton::setIconBadgeNumber(int number)
 {
-    if (number) {
-        const QString text = QString::number(number);
-        m_label->setText(text);
-
-        QSize size = m_label->sizeHint();
-        if (size.width() < size.height())
-            //Ensure we increase size by an even number of pixels
-            size.setWidth(size.height() + ((size.width() - size.height()) & 1));
-        m_label->resize(size);
-
-        //Do not show yet, we wait until the button has been resized
-    } else {
-        m_label->setText(QString());
-        m_label->hide();
-    }
+    QString text = (number ? QString::number(number) : QString());
+    m_badgeNumberLabel.setText(text);
     updateGeometry();
 }
 
@@ -770,32 +781,81 @@ OutputPaneManageButton::OutputPaneManageButton()
 {
     setFocusPolicy(Qt::NoFocus);
     setCheckable(true);
-    setStyleSheet(QLatin1String("QToolButton { border-image: url(:/core/images/panel_manage_button.png) 2 2 2 2;"
-                                " border-width: 2px 2px 2px 2px } "
-                                "QToolButton::menu-indicator { width:0; height:0 }"));
+    setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
 }
 
 QSize OutputPaneManageButton::sizeHint() const
 {
     ensurePolished();
-    return QSize(18, QApplication::globalStrut().height());
+    return QSize(numberAreaWidth(), QApplication::globalStrut().height());
 }
 
-void OutputPaneManageButton::paintEvent(QPaintEvent *event)
+void OutputPaneManageButton::paintEvent(QPaintEvent*)
 {
-    QToolButton::paintEvent(event); // Draw style sheet.
     QPainter p(this);
+    if (!creatorTheme()->flag(Theme::FlatToolBars)) {
+        static const QImage button(StyleHelper::dpiSpecificImageFile(QStringLiteral(":/utils/images/panel_manage_button.png")));
+        StyleHelper::drawCornerImage(button, &p, rect(), buttonBorderWidth, buttonBorderWidth, buttonBorderWidth, buttonBorderWidth);
+    }
     QStyle *s = style();
     QStyleOption arrowOpt;
     arrowOpt.initFrom(this);
-    arrowOpt.rect = QRect(5, rect().center().y() - 3, 9, 9);
+    arrowOpt.rect = QRect(6, rect().center().y() - 3, 8, 8);
     arrowOpt.rect.translate(0, -3);
     s->drawPrimitive(QStyle::PE_IndicatorArrowUp, &arrowOpt, &p, this);
     arrowOpt.rect.translate(0, 6);
     s->drawPrimitive(QStyle::PE_IndicatorArrowDown, &arrowOpt, &p, this);
 }
 
+BadgeLabel::BadgeLabel()
+{
+    m_font = QApplication::font();
+    m_font.setBold(true);
+    m_font.setPixelSize(11);
+}
+
+void BadgeLabel::paint(QPainter *p, int x, int y, bool isChecked)
+{
+    const QRectF rect(QRect(QPoint(x, y), m_size));
+    p->save();
+
+    p->setBrush(creatorTheme()->color(isChecked? Theme::BadgeLabelBackgroundColorChecked
+                                               : Theme::BadgeLabelBackgroundColorUnchecked));
+    p->setPen(Qt::NoPen);
+    p->setRenderHint(QPainter::Antialiasing, true);
+    p->drawRoundedRect(rect, m_padding, m_padding, Qt::AbsoluteSize);
+
+    p->setFont(m_font);
+    p->setPen(creatorTheme()->color(isChecked ? Theme::BadgeLabelTextColorChecked
+                                              : Theme::BadgeLabelTextColorUnchecked));
+    p->drawText(rect, Qt::AlignCenter, m_text);
+
+    p->restore();
+}
+
+void BadgeLabel::setText(const QString &text)
+{
+    m_text = text;
+    calculateSize();
+}
+
+QString BadgeLabel::text() const
+{
+    return m_text;
+}
+
+QSize BadgeLabel::sizeHint() const
+{
+    return m_size;
+}
+
+void BadgeLabel::calculateSize()
+{
+    const QFontMetrics fm(m_font);
+    m_size = fm.size(Qt::TextSingleLine, m_text);
+    m_size.setWidth(m_size.width() + m_padding * 1.5);
+    m_size.setHeight(2 * m_padding + 1); // Needs to be uneven for pixel perfect vertical centering in the button
+}
+
 } // namespace Internal
 } // namespace Core
-
-
