@@ -168,7 +168,7 @@ void CppSourceProcessor::addFrameworkPath(const ProjectPartHeaderPath &framework
         m_headerPaths.append(cleanFrameworkPath);
 
     const QDir frameworkDir(cleanFrameworkPath.path);
-    const QStringList filter = QStringList() << QLatin1String("*.framework");
+    const QStringList filter = QStringList("*.framework");
     foreach (const QFileInfo &framework, frameworkDir.entryInfoList(filter)) {
         if (!framework.isDir())
             continue;
@@ -448,7 +448,7 @@ void CppSourceProcessor::sourceNeeded(unsigned line, const QString &fileName, In
     }
 
     const QFileInfo info(absoluteFileName);
-    if (skipFileDueToSizeLimit(info))
+    if (fileSizeExceedsLimit(info, m_fileSizeLimitInMb))
         return; // TODO: Add diagnostic message
 
     // Otherwise get file contents
@@ -505,6 +505,11 @@ void CppSourceProcessor::sourceNeeded(unsigned line, const QString &fileName, In
     switchCurrentDocument(previousDocument);
 }
 
+void CppSourceProcessor::setFileSizeLimitInMb(int fileSizeLimitInMb)
+{
+    m_fileSizeLimitInMb = fileSizeLimitInMb;
+}
+
 Document::Ptr CppSourceProcessor::switchCurrentDocument(Document::Ptr doc)
 {
     const Document::Ptr previousDoc = m_currentDoc;
