@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef DEBUGGER_GDBENGINE_H
-#define DEBUGGER_GDBENGINE_H
+#pragma once
 
 #include <debugger/debuggerengine.h>
 
@@ -32,6 +31,7 @@
 #include <debugger/registerhandler.h>
 #include <debugger/watchhandler.h>
 #include <debugger/watchutils.h>
+#include <debugger/debuggeritem.h>
 #include <debugger/debuggertooltipmanager.h>
 
 #include <coreplugin/id.h>
@@ -44,6 +44,8 @@
 #include <QTime>
 #include <QTimer>
 
+#include <functional>
+
 namespace Debugger {
 namespace Internal {
 
@@ -64,8 +66,8 @@ class GdbEngine : public DebuggerEngine
     Q_OBJECT
 
 public:
-    explicit GdbEngine(const DebuggerRunParameters &runParameters);
-    ~GdbEngine();
+    explicit GdbEngine(bool useTerminal);
+    ~GdbEngine() override;
 
 private: ////////// General Interface //////////
     DebuggerEngine *cppEngine() override { return this; }
@@ -123,68 +125,46 @@ protected: ////////// Gdb Process Management //////////
     // Make sure to clean up everything before emitting this signal.
     void handleAdapterCrashed(const QString &msg);
 
-private slots:
+private:
     friend class GdbPlainEngine;
+    friend class GdbCoreEngine;
     void handleInterruptDeviceInferior(const QString &error);
     void handleGdbFinished(int exitCode, QProcess::ExitStatus exitStatus);
     void handleGdbError(QProcess::ProcessError error);
-    void readDebugeeOutput(const QByteArray &data);
     void readGdbStandardOutput();
     void readGdbStandardError();
+    void readDebuggeeOutput(const QByteArray &ba);
 
-private:
-    QTextCodec *m_outputCodec;
-    QTextCodec::ConverterState m_outputCodecState;
+    QTextCodec *m_gdbOutputCodec;
+    QTextCodec::ConverterState m_gdbOutputCodecState;
+    QTextCodec *m_inferiorOutputCodec;
+    QTextCodec::ConverterState m_inferiorOutputCodecState;
 
     QByteArray m_inbuffer;
     bool m_busy;
 
     // Name of the convenience variable containing the last
     // known function return value.
-    QByteArray m_resultVarName;
+    QString m_resultVarName;
 
-private: ////////// Gdb Command Management //////////
-
-    public: // Otherwise the Qt flag macros are unhappy.
-    enum GdbCommandFlag {
-        NoFlags = 0,
-        // The command needs a stopped inferior.
-        NeedsStop = 1,
-        // No need to wait for the reply before continuing inferior.
-        Discardable = 2,
-        // Callback expects ResultRunning instead of ResultDone.
-        RunRequest = 16,
-        // Callback expects ResultExit instead of ResultDone.
-        ExitRequest = 32,
-        // Auto-set inferior shutdown related states.
-        LosesChild = 64,
-        // Trigger breakpoint model rebuild when no such commands are pending anymore.
-        RebuildBreakpointModel = 128,
-        // This is a command that needs to be wrapped into -interpreter-exec console
-        ConsoleCommand = 512,
-        // This is the UpdateLocals commannd during which we ignore notifications
-        InUpdateLocals = 1024,
-        // This is a command using the python interface
-        PythonCommand = 2048
-    };
-    Q_DECLARE_FLAGS(GdbCommandFlags, GdbCommandFlag)
+protected: ////////// Gdb Command Management //////////
 
     void runCommand(const DebuggerCommand &command) override;
 
 private:
-    Q_SLOT void commandTimeout();
+    void commandTimeout();
     void setTokenBarrier();
 
     // Sets up an "unexpected result" for the following commeand.
-    void scheduleTestResponse(int testCase, const QByteArray &response);
+    void scheduleTestResponse(int testCase, const QString &response);
 
     QHash<int, DebuggerCommand> m_commandForToken;
     QHash<int, int> m_flagsForToken;
     int commandTimeoutTime() const;
     QTimer m_commandTimer;
 
-    QByteArray m_pendingConsoleStreamOutput;
-    QByteArray m_pendingLogStreamOutput;
+    QString m_pendingConsoleStreamOutput;
+    QString m_pendingLogStreamOutput;
 
     // This contains the first token number for the current round
     // of evaluation. Responses with older tokens are considers
@@ -202,13 +182,13 @@ private:
 
 private: ////////// Gdb Output, State & Capability Handling //////////
 protected:
-    Q_SLOT void handleResponse(const QByteArray &buff);
-    void handleAsyncOutput(const QByteArray &asyncClass, const GdbMi &result);
+    Q_INVOKABLE void handleResponse(const QString &buff);
+    void handleAsyncOutput(const QString &asyncClass, const GdbMi &result);
     void handleStopResponse(const GdbMi &data);
     void handleResultRecord(DebuggerResponse *response);
     void handleStop1(const GdbMi &data);
     void handleStop2(const GdbMi &data);
-    Q_SLOT void handleStop2();
+    void handleStop3();
     void resetCommandQueue();
 
     bool isSynchronous() const override { return true; }
@@ -255,14 +235,13 @@ private: ////////// Inferior Management //////////
     void handleExecuteJumpToLine(const DebuggerResponse &response);
     void handleExecuteRunToLine(const DebuggerResponse &response);
 
-    void maybeHandleInferiorPidChanged(const QString &pid);
-    void handleInfoProc(const DebuggerResponse &response);
     QString msgPtraceError(DebuggerStartMode sm);
 
 private: ////////// View & Data Stuff //////////
 
     void selectThread(ThreadId threadId) override;
     void activateFrame(int index) override;
+    void handleAutoContinueInferior();
 
     //
     // Breakpoint specific stuff
@@ -282,8 +261,8 @@ private: ////////// View & Data Stuff //////////
     void handleCatchInsert(const DebuggerResponse &response, Breakpoint bp);
     void handleBkpt(const GdbMi &bkpt, Breakpoint bp);
     void updateResponse(BreakpointResponse &response, const GdbMi &bkpt);
-    QByteArray breakpointLocation(const BreakpointParameters &data); // For gdb/MI.
-    QByteArray breakpointLocation2(const BreakpointParameters &data); // For gdb/CLI fallback.
+    QString breakpointLocation(const BreakpointParameters &data); // For gdb/MI.
+    QString breakpointLocation2(const BreakpointParameters &data); // For gdb/CLI fallback.
     QString breakLocation(const QString &file) const;
 
     //
@@ -291,7 +270,7 @@ private: ////////// View & Data Stuff //////////
     //
     protected:
     void loadSymbols(const QString &moduleName) override;
-    Q_SLOT void loadAllSymbols() override;
+    void loadAllSymbols() override;
     void loadSymbolsForStack() override;
     void requestModuleSymbols(const QString &moduleName) override;
     void requestModuleSections(const QString &moduleName) override;
@@ -311,8 +290,8 @@ private: ////////// View & Data Stuff //////////
     //
     // Register specific stuff
     //
-    Q_SLOT void reloadRegisters() override;
-    void setRegisterValue(const QByteArray &name, const QString &value) override;
+    void reloadRegisters() override;
+    void setRegisterValue(const QString &name, const QString &value) override;
     void handleRegisterListNames(const DebuggerResponse &response);
     void handleRegisterListing(const DebuggerResponse &response);
     void handleRegisterListValues(const DebuggerResponse &response);
@@ -327,7 +306,7 @@ private: ////////// View & Data Stuff //////////
     void fetchDisassemblerByCliPointMixed(const DisassemblerAgentCookie &ac);
     void fetchDisassemblerByCliRangeMixed(const DisassemblerAgentCookie &ac);
     void fetchDisassemblerByCliRangePlain(const DisassemblerAgentCookie &ac);
-    bool handleCliDisassemblerResult(const QByteArray &response, DisassemblerAgent *agent);
+    bool handleCliDisassemblerResult(const QString &response, DisassemblerAgent *agent);
 
     //
     // Source file specific stuff
@@ -357,38 +336,30 @@ protected:
     void handleThreadInfo(const DebuggerResponse &response);
     void handleThreadNames(const DebuggerResponse &response);
     DebuggerCommand stackCommand(int depth);
-    Q_SLOT void reloadStack();
-    Q_SLOT virtual void reloadFullStack() override;
-    virtual void loadAdditionalQmlStack() override;
-    void handleQmlStackTrace(const DebuggerResponse &response);
+    void reloadStack();
+    void reloadFullStack() override;
+    void loadAdditionalQmlStack() override;
     int currentFrame() const;
 
-    QList<GdbMi> m_currentFunctionArgs;
-
     //
     // Watch specific stuff
     //
     virtual void assignValueInDebugger(WatchItem *item,
         const QString &expr, const QVariant &value) override;
 
-    virtual void fetchMemory(MemoryAgent *agent, QObject *token,
-        quint64 addr, quint64 length) override;
+    void fetchMemory(MemoryAgent *agent, quint64 addr, quint64 length) override;
     void fetchMemoryHelper(const MemoryAgentCookie &cookie);
     void handleChangeMemory(const DebuggerResponse &response);
-    virtual void changeMemory(MemoryAgent *agent, QObject *token,
-        quint64 addr, const QByteArray &data) override;
+    void changeMemory(MemoryAgent *agent, quint64 addr, const QByteArray &data) override;
     void handleFetchMemory(const DebuggerResponse &response, MemoryAgentCookie ac);
 
-    virtual void watchPoint(const QPoint &) override;
-    void handleWatchPoint(const DebuggerResponse &response);
-
     void showToolTip();
 
     void handleVarAssign(const DebuggerResponse &response);
     void handleThreadGroupCreated(const GdbMi &result);
     void handleThreadGroupExited(const GdbMi &result);
 
-    Q_SLOT void createFullBacktrace();
+    void createFullBacktrace();
 
     void doUpdateLocals(const UpdateParameters &parameters) override;
     void handleFetchVariables(const DebuggerResponse &response);
@@ -400,15 +371,13 @@ protected:
     //
     void reloadDebuggingHelpers() override;
 
-    QString m_gdb;
-
     //
     // Convenience Functions
     //
-    QString errorMessage(QProcess::ProcessError error);
     void showExecutionError(const QString &message);
+    QString failedToStartMessage();
 
-    static QByteArray tooltipIName(const QString &exp);
+    static QString tooltipIName(const QString &exp);
 
     // For short-circuiting stack and thread list evaluation.
     bool m_stackNeeded;
@@ -418,15 +387,16 @@ protected:
     bool m_inUpdateLocals;
 
     // HACK:
-    QByteArray m_currentThread;
+    QString m_currentThread;
     QString m_lastWinException;
     QString m_lastMissingDebugInfo;
     bool m_terminalTrap;
-    bool m_temporaryStopPending;
     bool usesExecInterrupt() const;
     bool usesTargetAsync() const;
 
-    QHash<int, QByteArray> m_scheduledTestResponses;
+    DebuggerCommandSequence m_onStop;
+
+    QHash<int, QString> m_scheduledTestResponses;
     QSet<int> m_testCases;
 
     // Debug information
@@ -434,10 +404,6 @@ protected:
     void requestDebugInformation(const DebugInfoTask &task);
     DebugInfoTaskHandler *m_debugInfoTaskHandler;
 
-    // Indicates whether we had at least one full attempt to load
-    // debug information.
-    bool attemptQuickStart() const;
-    bool m_fullStartDone;
     bool m_systemDumpersLoaded;
 
     static QString msgGdbStopFailed(const QString &why);
@@ -446,16 +412,11 @@ protected:
     static QString msgInferiorSetupOk();
     static QString msgInferiorRunOk();
     static QString msgConnectRemoteServerFailed(const QString &why);
-    static QByteArray dotEscape(QByteArray str);
 
     void debugLastCommand() override;
     DebuggerCommand m_lastDebuggableCommand;
 
 protected:
-    virtual void write(const QByteArray &data);
-
-protected:
-    bool prepareCommand();
     void interruptLocalInferior(qint64 pid);
 
 protected:
@@ -466,7 +427,3 @@ protected:
 
 } // namespace Internal
 } // namespace Debugger
-
-Q_DECLARE_OPERATORS_FOR_FLAGS(Debugger::Internal::GdbEngine::GdbCommandFlags)
-
-#endif // DEBUGGER_GDBENGINE_H
