@@ -278,6 +278,11 @@ DeviceTester *IDevice::createDeviceTester() const
     return 0;
 }
 
+Utils::OsType IDevice::osType() const
+{
+    return Utils::OsTypeOther;
+}
+
 DeviceProcess *IDevice::createProcess(QObject * /* parent */) const
 {
     QTC_CHECK(false);
@@ -340,8 +345,10 @@ void IDevice::fromMap(const QVariantMap &map)
     if (optionsVariant.isValid())  // false for QtC < 3.4
         d->sshParameters.options = QSsh::SshConnectionOptions(optionsVariant.toInt());
 
-    d->freePorts = Utils::PortList::fromString(map.value(QLatin1String(PortsSpecKey),
-        QLatin1String("10000-10100")).toString());
+    QString portsSpec = map.value(PortsSpecKey).toString();
+    if (portsSpec.isEmpty())
+        portsSpec = "10000-10100";
+    d->freePorts = Utils::PortList::fromString(portsSpec);
     d->machineType = static_cast<MachineType>(map.value(QLatin1String(MachineTypeKey), DefaultMachineType).toInt());
     d->version = map.value(QLatin1String(VersionKey), 0).toInt();
 
@@ -404,9 +411,11 @@ void IDevice::setSshParameters(const QSsh::SshConnectionParameters &sshParameter
     d->sshParameters.hostKeyDatabase = DeviceManager::instance()->hostKeyDatabase();
 }
 
-Connection IDevice::toolControlChannel(const ControlChannelHint &) const
+QUrl IDevice::toolControlChannel(const ControlChannelHint &) const
 {
-    return HostName(d->sshParameters.host);
+    QUrl url;
+    url.setHost(d->sshParameters.host);
+    return url;
 }
 
 void IDevice::setFreePorts(const Utils::PortList &freePorts)
@@ -463,6 +472,4 @@ DeviceEnvironmentFetcher::DeviceEnvironmentFetcher()
 {
 }
 
-void *HostName::staticTypeId = &HostName::staticTypeId;
-
 } // namespace ProjectExplorer
